#include<stdio.h>
#include<stdlib.h>

#define Dim 3
#define Dim2 Dim*Dim

typedef enum {false, true} Bool;

typedef struct {
  int numbers[Dim2];
  int n_numbers;
} Domain;

int board[Dim2][Dim2] = 
  {
    {2,0,9,0,0,0,0,7,0},
    {0,5,0,0,0,0,0,0,0},
    {8,0,0,0,2,0,0,1,6},
    {0,8,1,4,0,0,7,2,0},
    {0,0,0,0,6,8,0,0,1},
    {0,0,0,0,0,7,0,9,0},
    {1,2,3,0,0,0,0,0,9},
    {0,0,0,0,4,0,0,0,0},
    {7,0,8,0,0,0,0,0,0},
  };

float GetRandomFloat() {
  return ((float) rand())/(((float)RAND_MAX)+1);
}

void PrintBoard(int b[Dim2][Dim2]) {
  
  int r,c;

  for (r=0; r<Dim2; r++) {
    for (c=0; c<Dim2; c++)
      if (b[r][c]>0) printf("%d", b[r][c]);
      else printf("-");
    printf("\n");
  }
}

void PrintMove(Domain move) {
  printf("Move: ");
  int i;
  for (i=0; i<move.n_numbers; i++)
    printf("%d ", move.numbers[i]);
  printf("\n");
}

Bool CanAssign(int b[Dim2][Dim2], int r, int c, int number) {
  int rr,cc;
  /* controlla che non ci sia nella stessa riga */
  for (cc=0; cc<Dim2; cc++)
    if (b[r][cc]==number) return false;
  // controlla che non ci sia nella stessa colonna
  for (rr=0; rr<Dim2; rr++)
    if (b[rr][c]==number) return false;
  // controlla che non ci sia nello stesso quadrato
  int r_min = r/Dim;
  int c_min = c/Dim;
  for (rr=Dim*r_min; rr<Dim*r_min+Dim; rr++)
    for (cc=Dim*c_min; cc<Dim*c_min+Dim; cc++)
      if (b[rr][cc]==number) return false;

  // altrimenti ok!
  return true;
}

Domain GetDomain(int b[Dim2][Dim2], int r, int c) {
  Domain d;
  d.n_numbers=0;

  int n;
  for (n=1; n<=Dim2; n++)
    if (CanAssign(b,r,c,n)) {
      d.numbers[d.n_numbers]=n;
      d.n_numbers++;
    }

  return d;
}

int SetBoard(int b[Dim2][Dim2], Bool fix[Dim2][Dim2]) {
  int r,c;
  int numbers_given=0;
  for (r=0; r<Dim2; r++)
    for (c=0; c<Dim2; c++)
      if (b[r][c]>0) {
	numbers_given++;
	fix[r][c]=true;
      }
      else
	fix[r][c]=false;
  return numbers_given;
}

void Solve(int b[Dim2][Dim2]) {

  Bool fix[Dim2][Dim2];

  int nFixed = SetBoard(b,fix);
  int assigned = nFixed;

  /*PrintBoard(fix);*/

  Domain move, best_move;

  int r,c;
  int r_best,c_best;

  while (assigned < Dim2*Dim2) {
   
    /*printf("Ass: %d\n", assigned); */

    best_move.n_numbers = Dim2+1;

    for (r=0; r<Dim2; r++)
      for (c=0; c<Dim2; c++)
	if (b[r][c]==0) {
	  move = GetDomain(b,r,c);
	  if (move.n_numbers < best_move.n_numbers) {
	    best_move.n_numbers = move.n_numbers;
	    int j;
	    for (j=0; j<best_move.n_numbers; j++)
	      best_move.numbers[j]=move.numbers[j];
	    r_best = r;
	    c_best = c;
	  }
	}

    printf("In row: %d col: %d\n", r_best, c_best);
    /*PrintMove(best_move);*/

    if (best_move.n_numbers>0) {
      /* fa una delle mosse in best_move */
      int rd = rand()%best_move.n_numbers;
      printf("Put number: %d\n", best_move.numbers[rd]);
      b[r_best][c_best] = best_move.numbers[rd];
      assigned++;
      printf("\n");
      PrintBoard(b);
      /*getchar();*/
    }
    else { /* BACKTRACK */
      int rr,cc;
      for (rr=0; rr<Dim2; rr++)
	for (cc=0; cc<Dim2; cc++)
	  if (!fix[rr][cc] && b[rr][cc]>0) {
	    if (GetRandomFloat() < 0.1) {
	      assigned--;
	      b[rr][cc]=0;
	    }
	  }
    }   
  }

}

int main() {

  srand(time(0));

  PrintBoard(board);
  Solve(board);
  printf("\n");
  printf("GREAT!! This is the solution:\n\n");
  PrintBoard(board);
}
